-- Database structure of nucleic acids.
-- 2011 Ondrej Cecak <ondrej@cecak.cz>, CVUT FEL
-- v1.0

--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
-- 
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
-- 
--  You should have received a copy of the GNU General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.


-- Entity Biomolecule
CREATE TABLE Biomolecule (
  id SERIAL PRIMARY KEY,
  pdbCode VARCHAR(4) UNIQUE
);

-- Entity BiomoleculeData
CREATE TABLE BiomoleculeData (
  id SERIAL PRIMARY KEY,
  biomoleculeId INTEGER UNIQUE,
  name TEXT,
  authors TEXT,
  method TEXT,
  dateDeposition DATE,
  dateRelease DATE,
  sequence TEXT
);

CREATE INDEX biomoleculeData_dateDeposition_idx ON BiomoleculeData(dateDeposition);
CREATE INDEX biomoleculeData_dateRelease_idx ON BiomoleculeData(dateRelease);

-- Entity MethodXRayData
CREATE TABLE MethodXRayData (
  biomoleculeDataId INTEGER PRIMARY KEY, 
  resolutionLow REAL, 
  resolutionHigh REAL, 
  method TEXT, 
  temperature REAL, 
  pH REAL, 
  reflectionsAll REAL, 
  reflectionsObs REAL
);

CREATE INDEX MethodXRayData_resolutionLow_idx ON MethodXRayData(resolutionLow);
CREATE INDEX MethodXRayData_resolutionHigh_idx ON MethodXRayData(resolutionHigh);
CREATE INDEX MethodXRayData_method_idx ON MethodXRayData(method);
CREATE INDEX MethodXRayData_temperature_idx ON MethodXRayData(temperature);
CREATE INDEX MethodXRayData_pH_idx ON MethodXRayData(pH);
CREATE INDEX MethodXRayData_reflectionsAll_idx ON MethodXRayData(reflectionsAll);
CREATE INDEX MethodXRayData_reflectionsObs_idx ON MethodXRayData(reflectionsObs);

-- Entity MethodNMRData
CREATE TABLE MethodNMRData (
  biomoleculeDataId INTEGER PRIMARY KEY, 
  fieldStrength REAL,
  numberOfModels INTEGER, 
  temperature REAL, 
  pressure TEXT, 
  pH REAL, 
  ionicStrenght TEXT
);

CREATE INDEX MethodNMRData_fieldStrength_idx ON MethodNMRData(fieldStrength);
CREATE INDEX MethodNMRData_numberOfModels_idx ON MethodNMRData(numberOfModels);
CREATE INDEX MethodNMRData_temperature_idx ON MethodNMRData(temperature);
CREATE INDEX MethodNMRData_pressure_idx ON MethodNMRData(pressure);
CREATE INDEX MethodNMRData_pH_idx ON MethodNMRData(pH);
CREATE INDEX MethodNMRData_ionicStrenght_idx ON MethodNMRData(ionicStrenght);

-- Entity Chain
CREATE TABLE Chain (
  id SERIAL PRIMARY KEY,
  biomoleculeId INTEGER, 
  name TEXT, 
  sequence TEXT, 
  lenght INTEGER
);

CREATE INDEX Chain_biomoleculeId_idx ON Chain(biomoleculeId);

-- Entity Residue
CREATE TABLE Residue (
  id SERIAL PRIMARY KEY,
  chainId INTEGER, 
  name TEXT, 
  position INTEGER
);

CREATE INDEX Residue_chainId_idx ON Residue(chainId);
CREATE INDEX Residue_position_idx ON Residue(position);

-- Entity Atom
CREATE TABLE Atom (
  id SERIAL PRIMARY KEY,
  residueId INTEGER, 
  element TEXT, 
  position INTEGER, 
  x REAL, 
  y REAL, 
  z REAL
);

CREATE INDEX Atom_residueId_idx ON Atom(residueId);

-- Entity CustomField
CREATE TABLE CustomField (
  id SERIAL PRIMARY KEY,
  biomoleculeId INTEGER,
  customName TEXT,
  customValue TEXT,
  customLinkId TEXT,
  customLinkKind TEXT
);

-- Entity StructureParameter
CREATE TABLE StructureParameter (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  shear REAL,
  stretch REAL,
  stagger REAL,
  buckle REAL, 
  propeller REAL,
  opening REAL,
  shift REAL,
  slide REAL,
  rise REAL,
  tilt REAL,
  roll REAL,
  twist REAL
);

CREATE INDEX StructureParameter_residueId_idx ON StructureParameter(residueId);
CREATE INDEX StructureParameter_shear_idx ON StructureParameter(shear);
CREATE INDEX StructureParameter_stretch_idx ON StructureParameter(stretch);
CREATE INDEX StructureParameter_stagger_idx ON StructureParameter(stagger);
CREATE INDEX StructureParameter_buckle_idx ON StructureParameter(buckle);
CREATE INDEX StructureParameter_propeller_idx ON StructureParameter(propeller);
CREATE INDEX StructureParameter_opening_idx ON StructureParameter(opening);
CREATE INDEX StructureParameter_shift_idx ON StructureParameter(shift);
CREATE INDEX StructureParameter_slide_idx ON StructureParameter(slide);
CREATE INDEX StructureParameter_rise_idx ON StructureParameter(rise);
CREATE INDEX StructureParameter_tilt_idx ON StructureParameter(tilt);
CREATE INDEX StructureParameter_roll_idx ON StructureParameter(roll);
CREATE INDEX StructureParameter_twist_idx ON StructureParameter(twist);

-- Entity OverlapArea
CREATE TABLE OverlapArea (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  i1_i2 REAL,
  i1_i2Ring REAL,
  i1_j2 REAL,
  i1_j2Ring REAL,
  j1_i2 REAL,
  j1_i2Ring REAL,
  j1_j2 REAL,
  j1_j2Ring REAL
);

CREATE INDEX OverlapArea_residueId_idx ON OverlapArea(residueId);
CREATE INDEX OverlapArea_i1_i2_idx ON OverlapArea(i1_i2);
CREATE INDEX OverlapArea_i1_i2Ring_idx ON OverlapArea(i1_i2Ring);
CREATE INDEX OverlapArea_i1_j2_idx ON OverlapArea(i1_j2);
CREATE INDEX OverlapArea_i1_j2Ring_idx ON OverlapArea(i1_j2Ring);
CREATE INDEX OverlapArea_j1_i2_idx ON OverlapArea(j1_i2); 
CREATE INDEX OverlapArea_j1_i2Ring_idx ON OverlapArea(j1_i2Ring);
CREATE INDEX OverlapArea_j1_j2_idx ON OverlapArea(j1_j2);
CREATE INDEX OverlapArea_j1_j2Ring_idx ON OverlapArea(j1_j2Ring);

-- Entity OriginMeanNormalVector
CREATE TABLE OriginMeanNormalVector (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  Ox REAL,
  Oy REAL,
  Oz REAL,
  Nx REAL,
  Ny REAL,
  Nz REAL
);

CREATE INDEX OriginMeanNormalVector_residueId_idx ON OriginMeanNormalVector(residueId);
CREATE INDEX OriginMeanNormalVector_Ox_idx ON OriginMeanNormalVector(Ox);
CREATE INDEX OriginMeanNormalVector_Oy_idx ON OriginMeanNormalVector(Oy);
CREATE INDEX OriginMeanNormalVector_Oz_idx ON OriginMeanNormalVector(Oz);
CREATE INDEX OriginMeanNormalVector_Nx_idx ON OriginMeanNormalVector(Nx);
CREATE INDEX OriginMeanNormalVector_Ny_idx ON OriginMeanNormalVector(Ny);
CREATE INDEX OriginMeanNormalVector_Nz_idx ON OriginMeanNormalVector(Nz);

-- Entity HelicalParameter
CREATE TABLE HelicalParameter (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  X_disp REAL,
  Y_disp REAL,
  h_Rise REAL,
  Incl REAL,
  Tip REAL,
  h_Twist REAL
);

CREATE INDEX HelicalParameter_residueId_idx ON HelicalParameter(residueId);
CREATE INDEX HelicalParameter_X_disp_idx ON HelicalParameter(X_disp);
CREATE INDEX HelicalParameter_Y_disp_idx ON HelicalParameter(Y_disp);
CREATE INDEX HelicalParameter_h_Rise_idx ON HelicalParameter(h_Rise);
CREATE INDEX HelicalParameter_Incl_idx ON HelicalParameter(Incl);
CREATE INDEX HelicalParameter_Tip_idx ON HelicalParameter(Tip);
CREATE INDEX HelicalParameter_h_Twist_idx ON HelicalParameter(h_Twist);

-- Entity StepClassification
CREATE TABLE StepClassification (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  Xp REAL,
  Yp REAL,
  Zp REAL,
  XpH REAL,
  YpH REAL,
  ZpH REAL
);

CREATE INDEX StepClassification_residueId_idx ON StepClassification(residueId);
CREATE INDEX StepClassification_Xp_idx ON StepClassification(Xp);
CREATE INDEX StepClassification_Yp_idx ON StepClassification(Yp);
CREATE INDEX StepClassification_Zp_idx ON StepClassification(Zp);
CREATE INDEX StepClassification_XpH_idx ON StepClassification(XpH);
CREATE INDEX StepClassification_YpH_idx ON StepClassification(YpH);
CREATE INDEX StepClassification_ZpH_idx ON StepClassification(ZpH);

-- Entity C1GlobalParameter
CREATE TABLE C1GlobalParameter (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  disp REAL,
  angle REAL,
  twist REAL,
  rise REAL
);

CREATE INDEX C1GlobalParameter_residueId_idx ON C1GlobalParameter(residueId);
CREATE INDEX C1GlobalParameter_disp_idx ON C1GlobalParameter(disp);
CREATE INDEX C1GlobalParameter_angle_idx ON C1GlobalParameter(angle);
CREATE INDEX C1GlobalParameter_twist_idx ON C1GlobalParameter(twist);
CREATE INDEX C1GlobalParameter_rise_idx ON C1GlobalParameter(rise);

-- Entity TorsionAngle
CREATE TABLE TorsionAngle (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  alpha REAL,
  beta REAL,
  gamma REAL,
  delta REAL,
  epsilon REAL,
  zeta REAL,
  chi REAL
);

CREATE INDEX TorsionAngle_residueId_idx ON TorsionAngle(residueId);
CREATE INDEX TorsionAngle_alpha_idx ON TorsionAngle(alpha);
CREATE INDEX TorsionAngle_beta_idx ON TorsionAngle(beta);
CREATE INDEX TorsionAngle_gamma_idx ON TorsionAngle(gamma);
CREATE INDEX TorsionAngle_delta_idx ON TorsionAngle(delta);
CREATE INDEX TorsionAngle_epsilon_idx ON TorsionAngle(epsilon);
CREATE INDEX TorsionAngle_zeta_idx ON TorsionAngle(zeta);
CREATE INDEX TorsionAngle_chi_idx ON TorsionAngle(chi);

-- Entity SugarConformationalParameter
CREATE TABLE SugarConformationalParameter (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  tm REAL,
  P REAL
);

CREATE INDEX SugarConformationalParameter_residueId_idx ON SugarConformationalParameter(residueId);
CREATE INDEX SugarConformationalParameter_tm_idx ON SugarConformationalParameter(tm);
CREATE INDEX SugarConformationalParameter_P_idx ON SugarConformationalParameter(P);

-- Entity PCVirtualBondDistance
CREATE TABLE PCVirtualBondDistance (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  PP REAL,
  C1C1 REAL
);

CREATE INDEX PCVirtualBondDistance_residueId_idx ON PCVirtualBondDistance(residueId);
CREATE INDEX PCVirtualBondDistance_PP_idx ON PCVirtualBondDistance(PP);
CREATE INDEX PCVirtualBondDistance_C1C1_idx ON PCVirtualBondDistance(C1C1);

-- Entity HelixRadiusRadialDisplacement 
CREATE TABLE HelixRadiusRadialDisplacement (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  P REAL,
  O4 REAL,
  C1 REAL
);

CREATE INDEX HelixRadiusRadialDisplacement_residueId_idx ON HelixRadiusRadialDisplacement(residueId);
CREATE INDEX HelixRadiusRadialDisplacement_P_idx ON HelixRadiusRadialDisplacement(P);
CREATE INDEX HelixRadiusRadialDisplacement_O4_idx ON HelixRadiusRadialDisplacement(O4);
CREATE INDEX HelixRadiusRadialDisplacement_C1_idx ON HelixRadiusRadialDisplacement(C1);

-- Entity PositionAndLocalHelicalAxisVector 
CREATE TABLE PositionAndLocalHelicalAxisVector (
  id SERIAL PRIMARY KEY,
  residueId INTEGER,
  Px REAL,
  Py REAL,
  Pz REAL,
  Hx REAL,
  Hy REAL,
  Hz REAL
);

CREATE INDEX PositionAndLocalHelicalAxisVector_residueId_idx ON PositionAndLocalHelicalAxisVector(residueId);
CREATE INDEX PositionAndLocalHelicalAxisVector_Px_idx ON PositionAndLocalHelicalAxisVector(Px);
CREATE INDEX PositionAndLocalHelicalAxisVector_Py_idx ON PositionAndLocalHelicalAxisVector(Py);
CREATE INDEX PositionAndLocalHelicalAxisVector_Pz_idx ON PositionAndLocalHelicalAxisVector(Pz);
CREATE INDEX PositionAndLocalHelicalAxisVector_Hx_idx ON PositionAndLocalHelicalAxisVector(Hx);
CREATE INDEX PositionAndLocalHelicalAxisVector_Hy_idx ON PositionAndLocalHelicalAxisVector(Hy);
CREATE INDEX PositionAndLocalHelicalAxisVector_Hz_idx ON PositionAndLocalHelicalAxisVector(Hz);

-- Entity ImportQueue
CREATE TABLE ImportQueue (
  pdbCode VARCHAR(4) PRIMARY KEY
);

-- Entity Import
CREATE TABLE Import (
  active SMALLINT,
  done INTEGER,
  started TIMESTAMP,
  updated TIMESTAMP,
  finished TIMESTAMP
);

